#!/usr/bin/env python3
# i75
# Copyright (C) 2023 Andrew Wilkinson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import requests
try:
    from typing import Dict, Optional
except ImportError:
    pass


def get(url: str,
        headers: Optional[Dict[str, str]] = None,
        data: Optional[str] = None,
        stream: bool = False) -> requests.Response:
    return requests.get(url, headers=headers, data=data, stream=stream)


def post(url: str,
         headers: Optional[Dict[str, str]] = None,
         data: Optional[str] = None,
         stream: bool = False) -> requests.Response:
    return requests.post(url, headers=headers, data=data, stream=stream)
